// Michael Klements
// RGB Colour Picker
// 3 January 2019
// www.the-diy-life.com

#include <LiquidCrystal.h>  //Include the required libraries
#include <Wire.h>
#include "Adafruit_TCS34725.h"

byte gammatable[256];       //Table to convert measured RGB values in more realistic visulisation colour on the RGB LED

Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_50MS, TCS34725_GAIN_4X);   //Setup the colour sensor through Adafruit library

LiquidCrystal lcd(15, 14, 16, 4, 5, 8, 7);  //Assign LCD pins

int pinLED = A0;      //Assign pins for the colour picker LED, push button and RGB LED
int pinButton = A1;
int redLED = 6;
int greenLED = 9;
int blueLED = 10;

#define commonAnode false   //Set up RGB LED common cathode or anode for gammatable - thanks PhilB for this gamma table

void setup()
{
  pinMode (pinLED, OUTPUT);     //Assign output pins
  pinMode (pinButton, INPUT);
  pinMode (redLED, OUTPUT);
  pinMode (greenLED, OUTPUT);
  pinMode (blueLED, OUTPUT);
  lcd.begin(16,2);              //Defines the number of characters and rows on our LCD
  lcd.clear();                  //Clear the screen
  lcd.setCursor(0,0);           //Set the cursor to first character, first row
  lcd.print("Colour Picker");   //Display this text
  analogWrite(redLED, 255);     //Routine to quickly fade change the LED from red to green to blue, just a visual effect
  for (int i=0; i<=255 ; i++)
  {
    analogWrite(redLED, 255-i);
    analogWrite(greenLED, i);
    delay(3);
  }
  for (int i=0; i<=255 ; i++)
  {
    analogWrite(greenLED, 255-i);
    analogWrite(blueLED, i);
    delay(3);
  }
  analogWrite(blueLED, 0);
  lcd.setCursor(0,1);
  if (tcs.begin())          //Connect to the colour sensor
  {
    lcd.print("Ready");
  } 
  else
  {
    lcd.print("Sensor Error");
    while (1);
  }
  digitalWrite(pinLED, LOW);  //Turn off the sensor's white LED
  for (int i=0; i<256; i++) //Set up the gamma table for RGB conversion
  {
    float x = i;
    x /= 255;
    x = pow(x, 2.5);
    x *= 255;
    if (commonAnode)
    {
      gammatable[i] = 255 - x;
    } 
    else
    {
      gammatable[i] = x;
    }
  }
}

void loop()
{
  if (digitalRead(pinButton) == LOW)  //If the button is pressed
  {
    float red, green, blue;       //Create variables for the measured RGB values
    digitalWrite(pinLED, HIGH);   //Turn the sensor LED on for measurement
    delay(500);                   //Delay to allow any movement from the button press to stop
    tcs.setInterrupt(false);      //Start measurement
    delay(60);                    //Takes 50ms to read
    tcs.getRGB(&red, &green, &blue);  //Get the required RGB values
    tcs.setInterrupt(true);
    delay(500);                   //This delay and delay at end just allow time for the button to be released, avoids repeated readings
    digitalWrite(pinLED, LOW);    //Turn off the sensor LED
    lcd.clear();                  //Clear the LCD display
    lcd.print("Colour Picked");   //Display the results
    lcd.setCursor(0,1);
    lcd.print("R:");
    lcd.print(int(red));
    lcd.print(" G:");
    lcd.print(int(green));
    lcd.print(" B:");
    lcd.print(int(blue));
    analogWrite(redLED, gammatable[(int)red]);      //Set the RGB LED to reflect the measured colour with gammatable conversion
    analogWrite(greenLED, gammatable[(int)green]);
    analogWrite(blueLED, gammatable[(int)blue]);
    delay(500);
  }
}
